CV_RegisterVar({
	name = "pvsc_allowcoreskins",
	defaultvalue = "Yes",
	flags = CV_NETVAR,
	possiblevalue = CV_YesNo,
	description = "Whether to allow players to use core characters."
})

local vanilla_skin_count = 63

local allowed_skins = {}

local function whitelist_core_skin(p,skinname)
	if not skinname then CONS_Printf(p, "pvsc_whitelistcoreskin <skin ID>: allow a core character to be used while core character are disabled.") return end
	if skins[skinname] == nil then return end
	allowed_skins[skinname] = true
end

local function blacklist_core_skin(p,skinname)
	if not skinname then CONS_Printf(p, "pvsc_unwhitelistcoreskin <skin ID>: de-whitelist a core character, returning it to normal usage rules.") return end
	if skins[skinname] == nil then return end
	allowed_skins[skinname] = nil
end

local function clear_whitelist(p)
	allowed_skins = {}
end

local function list_core_whitelist(p)
	if CV_FindVar("pvsc_allowcoreskins").value == 1 then
		CONS_Printf(p, "All core characters are currently allowed.")
		return
	end
	local count = 0
	for k, v in pairs(allowed_skins) do
		if v then
			count = count + 1
			CONS_Printf(p, skins[k].realname.." ("..k..")")
		end
	end
	if count == 0 then
		CONS_Printf(p, "No core characters are currently whitelisted.")
	end
end

COM_AddCommand("pvsc_whitelistcoreskin", whitelist_core_skin, COM_ADMIN)
COM_AddCommand("pvsc_unwhitelistcoreskin", blacklist_core_skin, COM_ADMIN)
COM_AddCommand("pvsc_clearcorewhitelist", clear_whitelist, COM_ADMIN)
COM_AddCommand("pvsc_listcorewhitelist", list_core_whitelist) -- normal players should be able to know this

addHook("PlayerThink", function(p)
	if CV_FindVar("pvsc_allowcoreskins").value == 1 then return end
	local is_vanilla_skin = #skins[p.skin] <= (vanilla_skin_count - 1) and not (allowed_skins[skins[p.skin].name])
	local is_fake_vanilla_skin
	if skins[p.mo.skin] ~= skins[p.skin] then
		is_fake_vanilla_skin = #skins[p.mo.skin] <= (vanilla_skin_count - 1)
	else
		is_fake_vanilla_skin = false -- don't bother if not using fakeskin
	end

	if (is_vanilla_skin == true) or (is_fake_vanilla_skin == true) then
		local random_skin
		if p._nvc_lastfakeskin then
			while true do
				random_skin = P_RandomRange(vanilla_skin_count, #skins)
				if skins[random_skin - 1].name ~= skins[p._nvc_lastfakeskin].name then break end
			end
		else
			random_skin = P_RandomRange(vanilla_skin_count, #skins)
		end

		if is_vanilla_skin then
			R_SetPlayerSkin(p, random_skin - 1)
			CONS_Printf(p, "Core characters are not allowed! You've been set to a random modded character.")
			if p.bot then
				p.skincolor = skins[random_skin - 1].prefcolor
				p.mo.color = skins[random_skin - 1].prefcolor
			end
		end
		if is_fake_vanilla_skin then
			p.mo.skin = skins[random_skin - 1].name
			p.kartspeed = skins[random_skin - 1].kartspeed
			p.kartweight = skins[random_skin - 1].kartweight
			p.charflags = skins[random_skin - 1].flags
			p._nvc_lastfakeskin = random_skin - 1
			if p.bot then
				p.skincolor = skins[random_skin - 1].prefcolor
				p.mo.color = skins[random_skin - 1].prefcolor
			end
		end
	end
end)
